import Taro, { useLoad, useDidShow, useReady, useDidHide, usePullDownRefresh, useShareAppMessage, useRouter, useShareTimeline } from '@tarojs/taro'
import { useMemo, useState } from 'react'
import type CustomTabBar from '@/custom-tab-bar'
import { View } from '@tarojs/components'
import { VideoInfo } from '@/constants/commentTypes'
import { videoListV2 } from '@/http/api'
import VideoSwiper from '@/components/VideoSwiper'
import { sharePageAppMessage } from '@/shareHelper'
import Route from '@/class/Route'
import { filterCardProps, pxToRpx, pageHeight } from '@/utils/index'
import { CATEGORY_PAGESOURCE } from '@/const/index'

// import { getTopSafeHeight, throttle, formatSecondsAsTime, getPreIds } from '@/utils'

// import Route from '@/class/Route'
import './index.less'

let pageNo = 1
let activeIndex = 0

export default function Index() {
    const page = useMemo(() => Taro.getCurrentInstance().page, []);
    const router = useRouter()
    const { params } = router || {}

    const [videoList, setVideoList] = useState<VideoInfo[]>([])
    const [refresherTriggered, setRefresherTriggered] = useState(false)
    const [needResumePlay, setNeedResumePlay] = useState(false)
    const [toSharePage, setToSharePage] = useState(false)
    const [flushList, setFlushList] = useState(false)
    const [botHeight, setBotHeight] = useState(0)
    const [showNoView, setShowNoView] = useState(false)

    useLoad(() => {
        const { redirect } = params
        let dur = 0
        if (redirect) {
            const route = new Route()
            route.push(decodeURIComponent(redirect))
            dur = 1000
            setToSharePage(true)
        }

        setTimeout(() => {
            fecthVideoListV2()
        }, dur);

        setBotHeight(pageHeight())

    })

    useDidShow(() => {
        const tabbar = Taro.getTabBar<CustomTabBar>(page)
        tabbar?.setSelected(0)

        if (videoList.length > 0 && toSharePage) {
            setToSharePage(false)
            setNeedResumePlay(!needResumePlay)
        }
    })

    useReady(() => {
    })

    useDidHide(() => {

    })

    usePullDownRefresh(() => {
        console.log('usePullDownRefresh', activeIndex)

        if (activeIndex ==0 ) {
            fecthVideoListV2(true)
        }


    })

    useShareAppMessage((shareRes) => {
        return sharePageAppMessage({
            video: videoList[activeIndex],
            activeIndex,
            router,
            shareRes
        })
    })

    // useShareTimeline( () =>{
    //     let video = videoList[activeIndex]
    //     return {
    //         title: video.title,
    //         query: '',
    //         imageUrl: video.coverImgPath
    //     }
    // })

    // const fecthVideoList = throttle(fecthVideoListV2)
    function fecthVideoListV2(cleanOldCardList = false) {
        if (refresherTriggered) {
            return 
        }
        setRefresherTriggered(true)

        if (cleanOldCardList)
            pageNo = 1

        Taro.$http.post(videoListV2, {
            categoryId: 55,
            pageNo,
            pageSize: 6,
            pageSource: 'pages/category'
        }).then((res: RequestType) => {
            const { code, data } = res
            if (code != 0)
                return

            let oldCardList = videoList

            if (cleanOldCardList) {
                oldCardList = []
            }

            pageNo++

            setVideoList([
                ...oldCardList,
                ...filterCardProps(data, CATEGORY_PAGESOURCE)
            ])
            if (cleanOldCardList) {
                setFlushList(true)
                setTimeout(() => {
                    setFlushList(false)
                }, 300);
            }

            setTimeout(() => {
                setRefresherTriggered(false)
            }, 300);



        }).catch(() => {
            setRefresherTriggered(false)
            if (videoList.length == 0) {
                setShowNoView(true)
            }
        })
    }

    function onSwiperRefresh() {
        showNoView && setShowNoView(false)

        if (activeIndex ==0 ) {
            fecthVideoListV2(true)
        }
        // fecthVideoListV2()

    }

    function onAnimationFinish(current) {
        activeIndex = current
        if (current >= videoList.length - 4) {
            fecthVideoListV2()
        }
    }

    return (
        <View className='category-page' style={{ height: 'calc(100% - ' +  `${Math.floor(pxToRpx(55))}px - ` + `${botHeight}px);` }}>
            {showNoView && <View className='noData' onClick={onSwiperRefresh}>请刷新重试</View>}
            <VideoSwiper
                list={videoList}
                flushList={flushList}
                needResumePlay={needResumePlay}
                onFinish={onAnimationFinish}
                onRefresh={onSwiperRefresh}
            />

        </View>
    )
}
