import { useState } from 'react'
import Taro, { useLoad, useDidShow, useReady, useDidHide, usePullDownRefresh, useShareAppMessage, useRouter } from '@tarojs/taro'
import { View, Image } from '@tarojs/components'
import { VideoInfo } from '@/constants/commentTypes'
import VideoSwiper from "@/components/VideoSwiper";
import { videoDetail, recommendSharePageList } from '@/http/api'
import { USER_SHARE_PAGESOURCE, RECOMMEND_PAGESOURCE } from '@/const/index'
import './share.less'
import { filterCardProps, pageHeight, pxToRpx, getMenuButtonBoundingClientRect } from '@/utils/index'
import { sharePageAppMessage, returnedCrowdClickReport } from '@/shareHelper'

let flushList = false
let needResumePlay = false

let activeIndex = 0

export default function Share() {

    const router = useRouter()
    const { params } = router || {}

    const [videoList, setVideoList] = useState<VideoInfo[]>([{
        id: +params.id!,
        title: decodeURIComponent(params.title || ''),
        coverImgPath: decodeURIComponent(params.shareImgPath || ''),
        shareTitle: decodeURIComponent(params.title || ''),
        videoPath: decodeURIComponent(params.videoPath || ''),
        recomTraceId: '',
        pageSource: USER_SHARE_PAGESOURCE,
        flowPool: '',
        user: {
            uid: 0,
            avatarUrl: decodeURIComponent(params.avatarUrl || ''),
            nickName: params.nickName || '',
        }
    }])
    const [botHeight, setBotHeight] = useState(0)
    const [backBtnTop, setBackBtnTop] = useState(0)


    useLoad(() => {
        console.log('pa', params)

        setBotHeight(pageHeight())
        setBackBtnTop(getMenuButtonBoundingClientRect().top)

        getVideoDetail()
        setTimeout(() => {
            getRecommendList()
        }, 300);

        returnedCrowdClickReport(params)
    })

    useReady(() => {

    })

    useDidShow(() => {

    })

    useDidHide(() => {

    })

    useShareAppMessage((shareRes) => {
        return sharePageAppMessage({
            video: videoList[activeIndex],
            activeIndex,
            router,
            shareRes
        })
    })

    function getVideoDetail() {
        let { id } = params
        console.log('share detail: ', Date.now())

        Taro.$http.post(videoDetail, {
            videoId: id,
            pageSource: 'lite-pages/user-videos-detail',
            rootPageSource: 'lite-pages/category',
        }).then((res: RequestType) => {
            const { code, data } = res
            if (code !== 0)
                return

            if (!data.videoPath || !data.id) {
                Taro.showToast({
                    title: '视频地址为空~!',
                    icon: 'error'
                })

                return
            }

            let videos = filterCardProps([data], USER_SHARE_PAGESOURCE)
            setVideoList(videos)
        })
    }

    function getRecommendList() {
        let { id } = params

        Taro.$http.post(recommendSharePageList, {
            currentVideoId: id
        }, { timeout: 5000 })
            .then(res => {
                const { code, data } = res
                if (code != 0) {
                    return
                }
                const recomList : VideoInfo[] = filterCardProps(data, RECOMMEND_PAGESOURCE)
                setVideoList([
                    ...videoList,
                    ...recomList
                ])
            })

    }

    function onAnimationFinish(current) {
        activeIndex = current
        if (current >= videoList.length - 4) {
            getRecommendList()
        }
    }
    function backBtnAction() {
        Taro.navigateBack()
    }

    function onRefresh() {
        
    }

    return (
        <View className='share-page'>
            <View className='backBtn' onClick={backBtnAction} style={{top: `${pxToRpx(backBtnTop)}px;`}}>
                <Image src='https://weapppiccdn.yishihui.com/wxicon/common/share_backbtn.png' />
            </View>
            <View className='swiper-bg' style={{ height: 'calc(100% - ' +  `${Math.floor(pxToRpx(55))}px - ` + `${botHeight}px);` }}>
                <VideoSwiper
                    list={videoList}
                    onFinish={onAnimationFinish}
                    needResumePlay={needResumePlay}
                    flushList={flushList} 
                    onRefresh={onRefresh}/>

            </View>
        </View>
    )
}
